<?
/* This file allows us at my company to import our LDAP data in the Tutos base.
 * Inspired from scheme.php !
 *
 * $Id: ldap_getdata.php,v 1.4.2.1 2002/09/17 08:12:12 gokohnert Exp $
 * $Author: gokohnert $
 *
 * Attention: this code is dedicated to my compay LDAP organisation
 * I'm not sure this will fit your LDAP structure, but you can use
 * that as an example.
 *
 * Doing this in a generic way is possible, but as i have just one example
 * of LDAP structure and no clue of general LDAP organisation, I just made
 * it work fine here.
 * Feel free to generalize this code if you know how to !
 *
 * !! Remove/Disable this script after running !!
 */
include("webelements.p3");
include("permission.p3");
include("mail.pinc");
include("table.pinc");

function ldap_import($dbconn, $action) {
  global $tutos, $lang, $current_user;

  /* Here we do the job
   * That means we connect to the LDAP server and fetch the data
   */

  $ds = ldap_connect($tutos[ldapauthserver]["host"],
                     $tutos[ldapauthserver]["port"]);

  if( $tutos[ldapauth_anonymous] ) {
    // this is an "anonymous" bind
    $r = ldap_bind($ds);
  } else {
    $r = ldap_bind($ds,
                   $tutos[ldapauthserver]["binddn"],
                   $tutos[ldapauthserver]["passwd"]);
  }

  /* We first fetch the locations of the differents company locations */

  // We do not want the top level node
  $loc_filter = "(!(ou=site))";
  $locdn  = "ou=administratif,ou=site,o=cvf";
  $fields = array("l", "ou", "description", "facsimiletelephonenumber",
		  "postaladdress", "postalcode");

  $sr   = ldap_search($ds, $locdn, $loc_filter, $fields);
  $info = ldap_get_entries($ds, $sr);

  for ($i=0; $i<$info["count"]; $i++) {
    $curloc = $info[$i]["ou"][0];

    // Dealing with our street format
    $street  = utf8_decode($info[$i]["postaladdress"][0]);
    $tmp     = explode("-", $street);
    $street1 = $tmp[0];
    $street2 = $tmp[1];

    $locs[$curloc] =
       array("lname"  => utf8_decode($info[$i]["description"][0]),
	     "city"   => utf8_decode($info[$i]["l"][0]),
	     "street1"=> $street1,
	     "street2"=> $street2,
	     "fax_1"  => utf8_decode($info[$i]["facsimiletelephonenumber"][0]),
	     "zip"    => utf8_decode($info[$i]["postalcode"][0]),
	     "country"=> "FR"
	     );
  } 

  /* We now fetch the list of dn to consider, $user_filter
   *
   * In our LDAP, we have some elements containing DN list, in order to
   * group LDAP entries in a high-level way.
   * So we are going to establish a list of user DN to consider when
   * importing data, based upon the grouping done in the LDAP.
   */
  $user_filter = "(|";

  // Where we find our groups
  $dn = "ou=administratif,ou=group,o=cvf";
  $fields = array("member");

  // The groups we consider
  $group_filter =
     "(|(cn=devprs)(cn=devbdx)(cn=devlil)(cn=devtrs)(cn=direction))";

  $sr   = ldap_search($ds, $dn, $group_filter, $fields);
  $info = ldap_get_entries($ds, $sr);

  for ($i=0; $i<$info["count"]; $i++) {
    for ($j=0; $j<$info[$i]["member"]["count"]; $j++) {
      $dn = $info[$i]["member"][$j]; 

      /* In order to use this DN as a LDAP search filter, we have
       * to transform it. The point is that we already know the basedn
       * from where we are gonna launch the search.
       */
      $tmp = explode(",", $dn);
      $user_filter .= "(&(".$tmp[0].")(".$tmp[1]."))";
    }
  }
  $user_filter .= ")";

  // echo $user_filter."<br>";

  // Here we prepare the actions to do, show or insert/update
  switch( $action ) {
  case "show":
    // Preparing the  show  (HTML rendering)
    echo "<table cellpadding=\"5\">";
    echo "<tr><th>uid</th><th>Name</th>";
    echo "<th>Dept.</th><th>mail</th><th>phone</th>";
    echo "</tr>";
    break;
    
  case "insert":
    // Preparing the insert/update: Company data
    echo "Cration de la structure";

    // No mails here !!
    $tutos[demo] = 0;

    $cvf = new company($dbconn);
    $cvf->name = "CVF";
    $msg .= $cvf->save();
    echo $lang[$cvf->getType()].":". $cvf->getFullName() ."<BR>\n";
   
    $dir = new department($dbconn);
    $dir->name = "Direction";
    $dir->company = &$cvf;
    $msg .= $dir->save();
    echo $lang[$dir->getType()].":". $dir->getFullName() ."<BR>\n";

    $dev = new department($dbconn);
    $dev->name = "Dveloppement";
    $dev->company = &$cvf;
    $msg .= $dev->save();
    echo $lang[$dev->getType()].":". $dev->getFullName() ."<BR>\n";

    $exp = new department($dbconn);
    $exp->name = "Exploitation";
    $exp->company = &$cvf;
    $msg .= $exp->save();
    echo $lang[$exp->getType()].":". $exp->getFullName() ."<BR>\n";
    flush();

    echo "<table cellpadding=\"5\">";
    echo "<tr><th>Address</th><th>User</th><th>Location</th>";
    echo "</tr>";
    break;

  case "update":
    echo "<table cellpadding=\"5\">";
    echo "<tr><th>Address</th><th>User</th><th>Location</th>";
    echo "</tr>";
    break;

  default:
    break;
  }

  /* Now we create our location objets. */
  $t = array("prs", "bdx", "lil", "trs");

  for($i=0; $i < count($t); $i++) {
    $l = new location($dbconn);

    $l->comapny = &$cvf;
    $l->lname   = $locs[$t[$i]]["lname"];
    $l->desc1   = $locs[$t[$i]]["lname"];
    $l->fax_1   = $locs[$t[$i]]["fax_1"];
    $l->street1 = $locs[$t[$i]]["street1"];
    $l->street2 = $locs[$t[$i]]["street2"];
    $l->city    = $locs[$t[$i]]["city"];
    $l->zip     = $locs[$t[$i]]["zip"];
    $l->country = $locs[$t[$i]]["country"];

    $t_locs[$t[$i]] = $l;
  }

  /* Now we fetch personnal informations */
  $userdn = "ou=administratif,ou=user,o=cvf";
  $fields = array("ou", "uid", "mail", "telephonenumber",
		  "homephone", "sn", "givenname");

  $sr = ldap_search($ds, $userdn, $user_filter, $fields);
  $info = ldap_get_entries($ds, $sr);

  /* Now we show/insert/update the LDAP users */
  for ($i=0; $i<$info["count"]; $i++) {
    $name  = utf8_decode($info[$i]["sn"][0])." ";
    $name .= utf8_decode($info[$i]["givenname"][0]);

    switch( $action ) {
    case "show":
      /* If not storing for real, just show the results */
      echo "<tr>";
      echo "<td>".$info[$i]["uid"][0]."</td>";
      echo "<td>".$name."</td>";
      echo "<td>".$locs[$info[$i]["ou"][0]]["lname"]."</td>";
      echo "<td>".$info[$i]["mail"][0]."</td>";
      echo "<td>".$info[$i]["telephonenumber"][0]."</td>";
      echo "</tr>";

      break;

    case "insert":
    case "update":
      /* This time we create tutos objects, then insert/update them
       * in the tutos data base.
       *
       * Note we have to explicilty look for the users in the tutos
       * database, cause the save choose to update only when it knows
       * the user id.
       */

      $f_name = utf8_decode($info[$i]["givenname"][0]);
    $l_name = utf8_decode($info[$i]["sn"][0]);
    $login  = $info[$i]["uid"][0];

    /* Get the passwd
     * In my company the passwd&account infos are separated from
     * the administrative ones, so i need to do one more LDAP request.
     */
    $pass_sr = ldap_search($ds, "ou=mail,ou=user,o=cvf",
			   "(uid=".$login.")", array("userPassword"));
    $pass_infos = ldap_get_entries($ds, $pass_sr);
    $passwd = $pass_infos[0]["userpassword"][0];

    $a = new tutos_address($dbconn);
    $u = new tutos_user($dbconn);

    // First we create the address
    $qa = "SELECT * FROM ". $a->tablename .
       " WHERE ".$dbconn->Like("f_name", $f_name).
       " AND ".  $dbconn->Like("l_name", $l_name);
    $ra = $dbconn->Exec($qa);

    if ( $ra->numrows() == 1 ) {
      // The user/address exists in the tutos data base
      $a->read_result($ra, 0);
      echo "<tr><td>".$a->getFullName()."</td>";

      $u->read($a->id, 0);
      $u->pw = $passwd;
      echo "<td>".$u->login."</td>";
    }
    else {
      $a->f_name  = $f_name;
      $a->l_name  = $l_name;
      $a->creator = &$current_user;
      $msg .= $a->save();
      echo "<tr><td>".$a->getFullName()."</td>";
      flush();

      $u->updatepw = 1;
      $u->login    = $login;
      $u->pw       = $passwd;
      $u->id       = $a->id;
      $msg .= $u->save();
      echo "<td>".$u->login."</td>";
      flush();
    }

    /* And now, the location stuff */
    $l = $t_locs[$info[$i]["ou"][0]];
    $mail = $info[$i]["mail"][0];

    $ql = "SELECT * FROM ". $l->tablename .
       " WHERE ".$dbconn->Like("email_1", $mail);
    $rl = $dbconn->Exec($ql);

    if ( $rl->numrows() == 1 ) {
      $l->read($rl->get(0, "id"));
    }

    // Update the location
    $l->email_1 = $mail;
    $l->phone_1 = $info[$i]["telephonenumber"][0];
    $l->phone_2 = $info[$i]["homephone"][0];
    $l->adr_id  = $a->id;

    $msg .= $l->save();
    echo "<td>".$l->lname."</td></tr>";

    /* In the case of an LDAP import, the user should not be
     * allowed to modify its entry
     */
    acl_lower($a, $u->id, $tutos[seeok]);
    acl_save($a);
    acl_lower($u, $u->id, $tutos[seeok]);
    acl_save($u);
    acl_lower($l, $u->id, $tutos[seeok]);
    acl_save($l);
    break;

    default:
      break;
    }
  } 
  echo "</table>";

  ldap_close($ds);
}

function my_ldap_walk_result($ds, $sr) {
  $info = ldap_get_entries($ds, $sr);
  echo $info["count"]." results"."<br>";

  for ($i=0; $i<$info["count"]; $i++) {
    echo "dn: " . $info[$i]["dn"] . "<br>";
    
    for ($ii=0; $ii<$info[$i]["count"]; $ii++) {
      $attrib = $info[$i][$ii];
      echo "&nbsp;&nbsp;" . $attrib . ":";
      echo $info[$i][$attrib][0];
      echo "<br>";
    }
  }
}

function navigate() {
  global $tutos, $lang, $current_user;

  echo "<TABLE>";
  echo "<TR><TD NOWRAP>";

  echo menulink("ldap_getdata.php?action=show", "View data");
  echo "<BR>\n";
  echo menulink("ldap_getdata.php?action=insert", "Insert data");
  echo "<BR>\n";
  echo menulink("ldap_getdata.php?action=update", "Update data");

  echo "<BR>\n";
  echo "</TD></TR>";

  HTML_Select();
}

function info($dbconn, $step, $action) {
  global $tutos, $lang, $current_user;

  echo "<SPAN CLASS=warn>If everything works disable this script</SPAN><BR>\n";
  echo "<TABLE BORDER=0 CELLPADDING=3 CELLSPACING=0>\n";
  echo "<TR>\n";
  echo " <TH COLSPAN=4>". $lang['AdminDBInfo'] ."</TH>\n";
  echo "</TR>\n";
  echo "<TR>\n";
  showfield("Database Type");
  echo " <TD>". $dbconn->GetType() ." ". $dbconn->openfunc ."</TD>\n";
  echo "</TR>\n";
  echo "<TR>\n";
  showfield($lang['AdminDBName']);
  echo " <TD>". $dbconn->db->name ."</TD>\n";
  showfield($lang['AdminDBUser']);
  echo " <TD>". $dbconn->db->user ."</TD>\n";
  echo "</TR>\n";
  echo "<TR>\n";
  showfield($lang['AdminDBHost']);
  echo " <TD>". $dbconn->db->host ."</TD>\n";
  showfield($lang['AdminDBPort']);
  echo " <TD>". $dbconn->db->port ."</TD>\n";
  echo "</TR>\n";
  echo "</TABLE>\n";
  flush();

  echo "<BR><BR>";

  // FIXME: use intl for the labels
  echo "<TABLE BORDER=0 CELLPADDING=3 CELLSPACING=0>\n";
  echo "<TR>\n";
  echo " <TH COLSPAN=4>". "LDAP Infos" ."</TH>\n";
  echo "</TR>\n";
  echo "<TR>\n";
  showfield("LDAP config");
  $use   = ( $tutos[ldapauth] ? "in use" : "not in use" );
  $crypt = ( $tutos[ldapauth_pw_enc] ? "in use" : "not in use" );
  $anon  = ( $tutos[ldapauth_anonymous] ? "anonymous" : "binddn" );
  echo ' <TD colspan="3" align="center">';
  echo "Auth ".$use . " - Crypt ".$crypt. " -  auth user is ".$anon;
  echo ' </TD>';
  echo "</TR>\n";
  echo "<TR>\n";
  showfield("Base DN");
  echo " <TD>". $tutos[ldapauthserver]['basedn']  ."</TD>\n";
  showfield("Bind DN");
  if( $tutos[ldapauth_anonymous] ) {
    echo " <TD>Anonymous</TD>\n";
  }
  else {
    echo " <TD>". $tutos[ldapauthserver]['binddn'] ."</TD>\n";
  }
  echo "</TR>\n";
  echo "<TR>\n";
  showfield($lang['AdminDBHost']);
  echo " <TD>". $tutos[ldapauthserver]['host'] ."</TD>\n";
  showfield($lang['AdminDBPort']);
  echo " <TD>". $tutos[ldapauthserver]['port']  ."</TD>\n";
  echo "</TR>\n";
  echo "</TABLE>\n";
  flush();

  echo "<br>";

  ldap_import($dbconn, $action);
  echo "<SPAN CLASS=warn>If everything works disable this script</SPAN><BR>\n";
}

ReadLang($lang);

if ( ! isset($step) ) {
  $step = 0;
}

HTML_Header("Import LDAP Data - ". $action);
if ( ! isset($id) ) {
  $id = 0;
}
$dbconn = DB_Open($id);
if ( $dbconn->gettype() == "Postgres" ) {
# No persistent connections here
  $dbconn->setPersistent(0);
}
if ( $dbconn->gettype() == "Interbase" ) {
# No persistent connections here
  $dbconn->setPersistent(0);
}

$current_user = new tutos_user($dbconn);
$current_user->f_name = "TUTOS";
$current_user->l_name = "(LDAP service)";
$current_user->admin = 1;
$msg = "";

HTML_Base1();
navigate();
HTML_Base2($msg);
info($dbconn,$step,$action);
HTML_Base3();

?>