<?
/**
 * This module allows you to import tasks definitions from CSV file.
 * It will certainly support more field later, but for now just allow
 * you to import your MSProject planning, for example.
 *
 * By Dimitri Fontaine <dfontaine@cvf.fr>
 *
 * $Id: task_import_csv.php,v 1.5.2.2 2002/08/28 16:34:22 gokohnert Exp $
 * $Author: gokohnert $
 *
 * @modulegroup task
 * @module task_import_csv
 * @package task
 */

include("webelements.p3");
include("permission.p3");
include("url.pinc");
include("task.pinc");
include("product.pinc");
include("timetrack.pinc");

/* Check if user is allowed to use it */
check_user();
loadmodules("task");

Function getTask(&$t, $name, $p_id) {
  global $dbconn;

  $qt = "SELECT * FROM ".$t->tablename.
     " WHERE ".$dbconn->Like("name", $name)." AND p_id = '".$p_id."'";
  $rt = $dbconn->Exec($qt);
	  
  if ( $rt->numrows() == 1 ) {
    $t->read_result($rt, 0);
    // echo "getTaskByName: $name [".$t->id."]<BR>\n";
  }
  else
    $t->setName($name);

  return $t;
}

Function import( &$p ) {
  global $tutos, $lang, $current_user, $dbconn;
  global $name, $index, $s_start, $s_end, $volume;
  global $HTTP_POST_VARS,$HTTP_POST_FILES;

  $gotourl="task_import_csv.php";
  $gotourl= addUrlParameter($gotourl, "p_id=". $p->id);

  if( isset($HTTP_POST_VARS['import']) ) {
    $import = $HTTP_POST_VARS['import'];
    if( ! isset($HTTP_POST_VARS['contents']) ) {
      $msg = $lang['Err0050'];
      $gotourl = addUrlParameter($gotourl, "msg=". UrlEncode($msg));
      $gotourl = addSessionKey($gotourl);

      Header("Status: 302 Moved Temporarily");
      Header("Location: ". getBaseUrl() . $gotourl);
      return;
    } else {
      $contents = $HTTP_POST_VARS['contents'];
	}
  }
  else {
    if( isset($HTTP_POST_FILES['userfile']) ) {
      $filename = $HTTP_POST_FILES['userfile']['tmp_name'];

      $fd = fopen($filename, "r");
      $contents = fread($fd, filesize($filename));
      fclose ($fd);
    }
  }
  // echo "<pre>$contents</pre>\n";
  $lines  = explode("\n", $contents);
  $header = $lines[0];

  // Guess the separator
  $seplist = array(":", "\t", ";", ",");
  $fnames  = $header;

  for($i=0; $i < count($seplist) && count($fnames) == 1; $i++) {
    $sep = $seplist[$i];
    $fnames = explode($sep, trim($header));
  }
  if( count($fnames) < 2 ) {
    $msg = $lang['Err0051'];
    $gotourl = addUrlParameter($gotourl, "msg=". UrlEncode($msg));
    $gotourl = addSessionKey($gotourl);

    Header("Status: 302 Moved Temporarily");
    Header("Location: ". getBaseUrl() . $gotourl);
    return;
  }

  // Prepare an association table
  $t_fields = array($name    => "name",
		    $s_start => "s_start",
		    $s_end   => "s_end",
		    $volume  => "volume",
		    $index   => "index"
		    );

  // Show what we read

  if( ! isset($import) ) {
    // Will have to confirm the import
    echo "<FORM name=\"import\" enctype=\"multipart/form-data\"";
    echo " action=\"task_import_csv.php\" method=\"POST\">\n";
    hiddenFormElements();
    echo "<INPUT TYPE=\"hidden\" name=\"p_id\" value=\"".$p->id."\">\n";
    echo "<INPUT TYPE=\"hidden\" name=\"contents\" value=\"".$contents."\">\n";

    echo "<INPUT TYPE=\"hidden\" name=\"name\"    value=\"".$name."\">\n";
    echo "<INPUT TYPE=\"hidden\" name=\"index\"   value=\"".$index."\">\n";
    echo "<INPUT TYPE=\"hidden\" name=\"s_start\" value=\"".$s_start."\">\n";
    echo "<INPUT TYPE=\"hidden\" name=\"s_end\"   value=\"".$s_end."\">\n";
    echo "<INPUT TYPE=\"hidden\" name=\"volume\"  value=\"".$volume."\">\n";
  }
  echo "<TABLE>\n";

  for($i=0; $i<count($lines); $i++) {
    $line = $lines[$i];

    if( trim($line) != "" ) {
      echo " <TR>\n";
      $fields = explode($sep, trim($line));

      $t = new task($dbconn);
      $t->id = -1;
      $t->creator = $current_user;
      $t->p_id = $p->id;

      for($j=0; $j < count($fields); $j++) {
	if( $i == 0 )
	  echo "<TH>".$fields[$j]."</TH>";

	else {
	  // Here we look what the field is
	  switch( $t_fields[$fnames[$j]] ) {
	  case "name": {
	    $name = $fields[$j];
	    $t->setName($name);

	    $val = $t->name;
	    break;
	  }

	  case "s_start": {
	    $s_start = new DateTime(reformat_date($fields[$j]));
	    $t->setSStart($s_start);
	    $val = $t->s_start->getYYYYMMDD();
	    break;
	  }

	  case "s_end": {
	    $s_end = new DateTime(reformat_date($fields[$j]));
	    $t->setSEnd($s_end);
	    $val = $t->s_end->getYYYYMMDD();
	    break;
	  }

	  case "volume": {
	    $vol = trim($fields[$j]);
	    if( ereg("([0-9]+).*(jour|day)", $vol, $regs) ) {
	      // FIXME: use a localized number of worked hours (8)
	      $t->setVolume(8*(int)$regs[1]);
	    }
	    elseif( ereg("([0-9]+).*(heure|hour)", $vol, $regs) ) {
	      $t->setVolume((int)$regs[1]);
	    }
	    else
	      $t->setVolume(0);

	    $val = $t->volume." h";
	    break;
	  }
	    /* Here it's a special case :
	     * MS Project exports dependency field as WBS Index
	     * The index is on the form: 1 1.1 1.2 2 2.1 2.1.1 2.1.2 2.2
	     * So we keep some info and associate a parent task
	     *
	     * This is based upon the fact that the tasks in the CSV file
	     * are sorted on this field.
	     */
	  case "index": {
	    $index = trim($fields[$j]);
	    $indexes[$index] = $t;

	    /* find the parent task with the index */
	    if( ereg("^([0-9.]*)\.[0-9]*$", $index, $regs) ) {
	      $p_task = $indexes[$regs[1]];
	      // echo "p_task: ".$p_task->name." [".$p_task->id."]<BR>\n";

	      $t->p_id   = $p_task->id;
	      $t->parent = getObject($dbconn, $p_task->id);

	      if( $p_task->id == -1 ) {
		$val = "$index : id(".$regs[1].")";
	      }
	      else {
		$val = "$index : <tt>".$p_task->id."</tt>";
	      }
	    }
	    else {
	      $t->parent = getObject($dbconn, $t->p_id);
	      $val = "$index";
	    }
	    break;
	  }

	  default:{
	    $val = "<i>Not Used</i> [".$t_fields[$fnames[$j]]."]".
	       " [".$fnames[$j].".]";
	    break;
	  }
	  }
	  // And we print it
	  echo "<TD>".$val."</TD>";
	}
      }
      echo " </TR>\n";

      /* Now we have registered the task fields, save the task
       * Only if the presentation page was confirmed, of course !
       */
      if( isset($import) ) {
	/* There's a MS Project way of doing things we have to
	 * care about.
	 * Milestones are exported as tasks with s_start == s_end and
	 * volume = 0.
	 * We won't import such tasks
	 */
	$is_ms_milestone =
	   ( $t->s_start->getYYYYMMDD() == $t->s_end->getYYYYMMDD() )
	   && $t->volume == 0;

	if( $t->name != "" && ! $is_ms_milestone ) {
	  $dbconn->Begin("WORK");
	  $t->save();
	  $dbconn->Commit("WORK");

	  // We need to get the task id
	  $t = getTask(&$t, $name, $t->p_id);

	  // Let the $indexes table up-to-date
	  $indexes[$index] = $t;

	  //echo "saved ($index): ".$t->name." [<tt>".$t->id."</tt>]<BR>\n";
	}
      }
    }
  }
  if( ! isset($import) ) {
    echo " <TR><TD colspan=\"".count($fields)."\" align=\"center\">";
    echo "<HR><BR>\n";
    echo "<INPUT type=\"submit\" name=\"import\"";
    echo " value=\"".$lang['ImportConfirm']."\">";
    echo " </TD></TR>\n";
  }
  echo "</TABLE>";
  echo "</FORM>";
}

/* The date format I had here is the one used by french version of MSProject.
 * As I'm not sure the format DD/MM/YY is widely used, i've prefered not to
 * add it to the DateTime class.
 */
Function reformat_date($date) {
  $datestr = explode(" ", $date);
  if( ereg("^([0-3][0-9])/([0-1][0-9])/([0-9]{2})$", $datestr[1], $regs) ) {
    // date format DD/MM/YY, transform to YYYYMMDD
    $d = "20".$regs[3].$regs[2].$regs[1];
  }
  else
    $d = $datestr[1];

  return $d;
}



Function info(&$p) {
  global $tutos, $lang, $current_user;

  echo "<FORM name=\"import\" enctype=\"multipart/form-data\"";
  echo " action=\"task_import_csv.php\" method=\"POST\">\n";
  hiddenFormElements();
  echo "<INPUT TYPE=\"hidden\" name=\"p_id\" value=\"".$p->id."\">";
  

  echo "<TABLE CLASS=single BORDER=0 CELLSPACING=0>\n";
  echo " <TR><TH COLSPAN=4>";
  echo "Import CSV (". $lang[$p->getType()] ." ". menulink($p->getUrl(),$p->getFullName()) .")";
  echo " </TH></TR>\n";

  echo " <TR>\n";
  echo "  <TD class=\"desc\">".$lang['ImportSrcFile']."</TD>\n";
  echo "  <TD><INPUT type=\"file\" accept=\"*.csv\" name=\"userfile\"></TD>\n";
  echo " </TR>\n";

  echo " <TR><TD colspan=\"2\"><HR></TD></TR>\n";
  echo " <TR><TH colspan=\"2\">".$lang['ImportColNames']."</TH></TR>\n";

  echo " <TR>\n";
  echo "  <TD class=\"desc\">".$lang['TaskName']."</TD>\n";
  echo "  <TD><INPUT name=\"name\"></TD>\n";
  echo " </TR>\n";

  echo " <TR>\n";
  echo "  <TD class=\"desc\">".$lang['ImportIndex']."</TD>\n";
  echo "  <TD><INPUT name=\"index\"></TD>\n";
  echo " </TR>\n";

  /*
  echo " <TR>\n"; 
  echo "  <TD class=\"desc\">".$lang['TaskState']."</TD>\n";
  echo "  <TD><INPUT name=\"state\"></TD>\n";
  echo " </TR>\n";
  */

  /*
  echo " <TR>\n";
  echo "  <TD class=\"desc\">".$lang['TaskDesc']."</TD>\n";
  echo "  <TD><INPUT name=\"desc\"></TD>\n";
  echo " </TR>\n";
  */

  echo " <TR>\n";
  echo "  <TD class=\"desc\">".$lang['TaskVolumeFull']."</TD>\n";
  echo "  <TD><INPUT name=\"volume\"></TD>\n";
  echo " <TR>\n";

  /*
  echo " <TR>\n";
  echo "  <TD class=\"desc\">".$lang['TaskVolumeDone']."</TD>\n";
  echo "  <TD><INPUT name=\"volume_done\"></TD>\n";
  echo " <TR>\n";
  */

  echo " <TR>\n";
  echo "  <TD class=\"desc\">".$lang['TaskS_Start']."</TD>\n";
  echo "  <TD><INPUT name=\"s_start\"></TD>\n";
  echo " <TR>\n";

  echo " <TR>\n";
  echo "  <TD class=\"desc\">".$lang['TaskS_End']."</TD>\n";
  echo "  <TD><INPUT name=\"s_end\"></TD>\n";
  echo " <TR>\n";

  if ( ($p->mod_ok()) || ($p->id == -1) ) {
    echo "<TR>\n";
    echo "  <TD colspan=\"2\" align=\"center\">";
    echo "<INPUT type=\"submit\" name=\"valid\" value=\"".
      $lang['ImportValid']."\"></TD>\n";
    echo "</TR>\n";
  }
  echo "</TABLE>\n";
  echo "</FORM>\n";
}

Function navigate() {
  global $tutos, $lang;

  echo "<TABLE>";
  echo "<TR><TD>";


  echo "</TD></TR>";

  HTML_Select();
}

HTML_Header($lang['TaskImport']);
$stop = 0;

if ( isset($HTTP_POST_VARS['p_id']) ) {
  $obj = GetObject($dbconn, $HTTP_POST_VARS['p_id']);
} else if ( isset($HTTP_GET_VARS['p_id']) ) {
  $obj = GetObject($dbconn, $HTTP_GET_VARS['p_id']);
} else {
  $msg .= $lang['Err0048'];
  $stop = 1;
  $obj = -1;
}

HTML_Base1();
navigate();
HTML_Base2($msg);

if ( $stop != 1 ) {
  if( isset($HTTP_POST_VARS['valid']) || isset($HTTP_POST_VARS['import']) ) {
    import($obj);
  }
  else {
    info($obj);
  }
}

HTML_Base3();

$dbconn->Close();
?>
